package com.nullchat.network;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;
import com.nullchat.service.ChatMessage;
import com.nullchat.service.MessageReceiver;

public class NullChatClient implements Runnable {

	public final String TAG = "NullChatClient";
	public static final int STATE_NONE = 0;
	public static final int STATE_CONNECTING = 1;
	public static final int STATE_HANDSHAKING = 2;
	public static final int STATE_CONNECTED = 3;
	public static final int STATE_DISCONNECTED = 4;
	private int state = STATE_NONE;
	private final static String[] HANDSHAKE_TAG = {
			"<handshake version=4 token=", "/>" };
	private String token = "0b2203acf7f9d9d754fadbaee34ef42b";
	private int port;
	private String server;
	private String tokenPage;
	private Socket s;
	private MessageReceiver mr;
	NullChatStream ncs;
	private boolean useTokenPage = false;

	public NullChatClient(String server, int port, String tokenPage) {
		this.server = server;
		this.port = port;
		useTokenPage = true;
		this.tokenPage = tokenPage;
		Logger.getLogger(TAG).log(
				Level.INFO,
				"NullChatClient(): server=" + server + " port=" + port
						+ " tokenPage=" + tokenPage);
	}

	public NullChatClient(String server, int port) {
		this.server = server;
		this.port = port;
		useTokenPage = false;
	}

	public void setMessageReceiver(MessageReceiver mr) {
		this.mr = mr;
	}

	public int getState() {
		return state;
	}

	@Override
	public void run() {
		Logger.getLogger(TAG).log(Level.INFO, "run()");
		state = STATE_CONNECTING;
		if (connect()) {
			state = STATE_CONNECTED;
			Logger.getLogger(TAG).log(Level.INFO, "run(): receive()");
			receive();
		} else
			disconnect();
	}

	private boolean connect() {
		if (useTokenPage) {
			mr.message(new ChatMessage("Getting token..."));
			String t = getToken();
			token = t != null ? t : token;
		}

		mr.message(new ChatMessage("Connecting..."));
		try {
			s = new Socket(server, port);
		} catch (UnknownHostException e) {
			mr.message(new ChatMessage("Unknown server host"));
			return false;
		} catch (IOException e) {
			mr.message(new ChatMessage("Can't connect to server"));
			return false;
		}

		String handshake = HANDSHAKE_TAG[0] + token + HANDSHAKE_TAG[1];
		mr.message(new ChatMessage("Handshaking..."));
		send(new ChatMessage(handshake));
		return true;
	}

	private String getToken() {
		URL tokenURL = null;
		InputStream input = null;
		byte[] bytes = null;
		int n = -1;

		try {
			tokenURL = new URL(tokenPage);
		} catch (MalformedURLException e) {
			mr.message(new ChatMessage("Bad token page"));
			return null;
		}

		try {
			input = tokenURL.openStream();
			byte[] buffer = new byte[4096];
			ByteArrayOutputStream baos = new ByteArrayOutputStream(4096);

			while ((n = input.read(buffer)) != -1) {
				if (n > 0) {
					baos.write(buffer, 0, n);
				}
			}
			bytes = baos.toByteArray();
		} catch (IOException ex) {
			Logger.getLogger(TAG).log(Level.INFO, "getToken() ", ex);
			return null;
		}

		String r = new String(bytes, 0, bytes.length);
		int tStart = r.indexOf("token=", 0) + 6;
		int tEnd = tStart + 32;
		String token = r.substring(tStart, tEnd).trim();
		return token;
	}
	
	private void receive() {
		try {
			ncs = new NullChatStream(s.getInputStream());
			while (state == STATE_CONNECTED) {
				ChatMessage cm = ncs.readMessage();
				mr.message(cm);
			}
		} catch (IOException ex) {
			disconnect();
			return;
		}
	}

	public void send(ChatMessage cm) {
		try {
			s.getOutputStream().write(cm.getBytes());
		} catch (IOException ex) {
			Logger.getLogger(TAG).log(Level.SEVERE,
					"send(): ", ex);
			disconnect();
		}
	}

	public void disconnect() {
		if (state != STATE_DISCONNECTED) {
			state = STATE_DISCONNECTED;
			try {
				ncs.close();
				s.close();
			} catch (Exception ex) {
				Logger.getLogger(TAG).log(
						Level.SEVERE, "disconnect(): ", ex);
			}
			mr.clientDisconnected();
		}
	}

}
